product_map = {
    1: "LIFX Original 1000",
    3: "LIFX Color 650",
    10: "LIFX White 800 (Low Voltage)",
    11: "LIFX White 800 (High Voltage)",
    15: "LIFX Color 1000",
    18: "LIFX White 900 BR30 (Low Voltage)",
    19: "LIFX White 900 BR30 (High Voltage)",
    20: "LIFX Color 1000 BR30",
    22: "LIFX Color 1000",
    27: "LIFX A19",
    28: "LIFX BR30",
    29: "LIFX A19 Night Vision",
    30: "LIFX BR30 Night Vision",
    31: "LIFX Z",
    32: "LIFX Z",
    36: "LIFX Downlight",
    37: "LIFX Downlight",
    38: "LIFX Beam",
    39: "LIFX Downlight White to Warm",
    40: "LIFX Downlight",
    43: "LIFX A19",
    44: "LIFX BR30",
    45: "LIFX A19 Night Vision",
    46: "LIFX BR30 Night Vision",
    49: "LIFX Mini Color",
    50: "LIFX Mini White to Warm",
    51: "LIFX Mini White",
    52: "LIFX GU10",
    53: "LIFX GU10",
    55: "LIFX Tile",
    57: "LIFX Candle",
    59: "LIFX Mini Color",
    60: "LIFX Mini White to Warm",
    61: "LIFX Mini White",
    62: "LIFX A19",
    63: "LIFX BR30",
    64: "LIFX A19 Night Vision",
    65: "LIFX BR30 Night Vision",
    66: "LIFX Mini White",
    68: "LIFX Candle",
    70: "LIFX Switch",
    71: "LIFX Switch",
    81: "LIFX Candle White to Warm",
    82: "LIFX Filament Clear",
    85: "LIFX Filament Amber",
    87: "LIFX Mini White",
    88: "LIFX Mini White",
    89: "LIFX Switch",
    90: "LIFX Clean",
    91: "LIFX Color",
    92: "LIFX Color",
    93: "LIFX A19 US",
    94: "LIFX BR30",
    96: "LIFX Candle White to Warm",
    97: "LIFX A19",
    98: "LIFX BR30",
    99: "LIFX Clean",
    100: "LIFX Filament Clear",
    101: "LIFX Filament Amber",
    109: "LIFX A19 Night Vision",
    110: "LIFX BR30 Night Vision",
    111: "LIFX A19 Night Vision",
    112: "LIFX BR30 Night Vision Intl",
    113: "LIFX Mini WW US",
    114: "LIFX Mini WW Intl",
    115: "LIFX Switch",
    116: "LIFX Switch",
    117: "LIFX Z US",
    118: "LIFX Z Intl",
    119: "LIFX Beam US",
    120: "LIFX Beam Intl",
    121: "LIFX Downlight Intl",
    122: "LIFX Downlight US",
    123: "LIFX Color US",
    124: "LIFX Colour Intl",
    125: "LIFX White to Warm US",
    126: "LIFX White to Warm Intl",
    127: "LIFX White US",
    128: "LIFX White Intl",
    129: "LIFX Color US",
    130: "LIFX Colour Intl",
    131: "LIFX White To Warm US",
    132: "LIFX White To Warm Intl",
    133: "LIFX White US",
    134: "LIFX White Intl",
    135: "LIFX GU10 Color US",
    136: "LIFX GU10 Colour Intl",
    137: "LIFX Candle Color US",
    138: "LIFX Candle Colour Intl",
    141: "LIFX Neon US",
    142: "LIFX Neon Intl",
    143: "LIFX String US",
    144: "LIFX String Intl",
    161: "LIFX Outdoor Neon US",
    162: "LIFX Outdoor Neon Intl",
    163: "LIFX A19 US",
    164: "LIFX BR30 US",
    165: "LIFX A19 Intl",
    166: "LIFX BR30 Intl",
    167: "LIFX Downlight",
    168: "LIFX Downlight",
    169: "LIFX A21 1600lm US",
    170: "LIFX A21 1600lm Intl",
    171: "LIFX Round Spot US",
    173: "LIFX Round Path US",
    174: "LIFX Square Path US",
    175: "LIFX PAR38 US",
    176: "LIFX Ceiling US",
    177: "LIFX Ceiling Intl",
    178: "LIFX Downlight US",
    179: "LIFX Downlight US",
    180: "LIFX Downlight US",
    181: "LIFX Color US",
    182: "LIFX Colour Intl",
    185: "LIFX Candle Color US",
    186: "LIFX Candle Colour Intl",
    187: "LIFX Candle Color US",
    188: "LIFX Candle Colour Intl",
    201: 'LIFX Ceiling 13x26" US',
    202: 'LIFX Ceiling 13x26" Intl',
    203: "LIFX String US",
    204: "LIFX String Intl",
    205: "LIFX Indoor Neon US",
    206: "LIFX Indoor Neon Intl",
    213: "LIFX Permanent Outdoor US",
    214: "LIFX Permanent Outdoor Intl",
    215: "LIFX Candle Color US",
    216: "LIFX Candle Colour Intl",
    217: "LIFX Tube US",
    218: "LIFX Tube Intl",
    219: "LIFX Luna US",
    220: "LIFX Luna Intl",
    221: "LIFX Round Spot Intl",
    222: "LIFX Round Path Intl",
    223: "LIFX Downlight US",
    224: "LIFX Downlight Intl",
    225: "LIFX PAR38 INTL",
}

features_map = {
    1: {
        "buttons": False,
        "chain": False,
        "color": True,
        "extended_multizone": False,
        "hev": False,
        "infrared": False,
        "matrix": False,
        "max_kelvin": 9000,
        "min_kelvin": 2500,
        "multizone": False,
        "relays": False,
    },
    3: {
        "buttons": False,
        "chain": False,
        "color": True,
        "extended_multizone": False,
        "hev": False,
        "infrared": False,
        "matrix": False,
        "max_kelvin": 9000,
        "min_kelvin": 2500,
        "multizone": False,
        "relays": False,
    },
    10: {
        "buttons": False,
        "chain": False,
        "color": False,
        "extended_multizone": False,
        "hev": False,
        "infrared": False,
        "matrix": False,
        "max_kelvin": 6500,
        "min_kelvin": 2700,
        "multizone": False,
        "relays": False,
    },
    11: {
        "buttons": False,
        "chain": False,
        "color": False,
        "extended_multizone": False,
        "hev": False,
        "infrared": False,
        "matrix": False,
        "max_kelvin": 6500,
        "min_kelvin": 2700,
        "multizone": False,
        "relays": False,
    },
    15: {
        "buttons": False,
        "chain": False,
        "color": True,
        "extended_multizone": False,
        "hev": False,
        "infrared": False,
        "matrix": False,
        "max_kelvin": 9000,
        "min_kelvin": 2500,
        "multizone": False,
        "relays": False,
    },
    18: {
        "buttons": False,
        "chain": False,
        "color": False,
        "extended_multizone": False,
        "hev": False,
        "infrared": False,
        "matrix": False,
        "max_kelvin": 9000,
        "min_kelvin": 2500,
        "multizone": False,
        "relays": False,
    },
    19: {
        "buttons": False,
        "chain": False,
        "color": False,
        "extended_multizone": False,
        "hev": False,
        "infrared": False,
        "matrix": False,
        "max_kelvin": 9000,
        "min_kelvin": 2500,
        "multizone": False,
        "relays": False,
    },
    20: {
        "buttons": False,
        "chain": False,
        "color": True,
        "extended_multizone": False,
        "hev": False,
        "infrared": False,
        "matrix": False,
        "max_kelvin": 9000,
        "min_kelvin": 2500,
        "multizone": False,
        "relays": False,
    },
    22: {
        "buttons": False,
        "chain": False,
        "color": True,
        "extended_multizone": False,
        "hev": False,
        "infrared": False,
        "matrix": False,
        "max_kelvin": 9000,
        "min_kelvin": 2500,
        "multizone": False,
        "relays": False,
    },
    27: {
        "buttons": False,
        "chain": False,
        "color": True,
        "extended_multizone": False,
        "hev": False,
        "infrared": False,
        "matrix": False,
        "max_kelvin": 9000,
        "min_kelvin": 1500,
        "multizone": False,
        "relays": False,
    },
    28: {
        "buttons": False,
        "chain": False,
        "color": True,
        "extended_multizone": False,
        "hev": False,
        "infrared": False,
        "matrix": False,
        "max_kelvin": 9000,
        "min_kelvin": 1500,
        "multizone": False,
        "relays": False,
    },
    29: {
        "buttons": False,
        "chain": False,
        "color": True,
        "extended_multizone": False,
        "hev": False,
        "infrared": True,
        "matrix": False,
        "max_kelvin": 9000,
        "min_kelvin": 1500,
        "multizone": False,
        "relays": False,
    },
    30: {
        "buttons": False,
        "chain": False,
        "color": True,
        "extended_multizone": False,
        "hev": False,
        "infrared": True,
        "matrix": False,
        "max_kelvin": 9000,
        "min_kelvin": 1500,
        "multizone": False,
        "relays": False,
    },
    31: {
        "buttons": False,
        "chain": False,
        "color": True,
        "extended_multizone": False,
        "hev": False,
        "infrared": False,
        "matrix": False,
        "max_kelvin": 9000,
        "min_kelvin": 2500,
        "multizone": True,
        "relays": False,
    },
    32: {
        "buttons": False,
        "chain": False,
        "color": True,
        "extended_multizone": True,
        "hev": False,
        "infrared": False,
        "matrix": False,
        "max_kelvin": 9000,
        "min_ext_mz_firmware": 1532997580,
        "min_ext_mz_firmware_components": [2, 77],
        "min_kelvin": 1500,
        "multizone": True,
        "relays": False,
    },
    36: {
        "buttons": False,
        "chain": False,
        "color": True,
        "extended_multizone": False,
        "hev": False,
        "infrared": False,
        "matrix": False,
        "max_kelvin": 9000,
        "min_kelvin": 1500,
        "multizone": False,
        "relays": False,
    },
    37: {
        "buttons": False,
        "chain": False,
        "color": True,
        "extended_multizone": False,
        "hev": False,
        "infrared": False,
        "matrix": False,
        "max_kelvin": 9000,
        "min_kelvin": 1500,
        "multizone": False,
        "relays": False,
    },
    38: {
        "buttons": False,
        "chain": False,
        "color": True,
        "extended_multizone": True,
        "hev": False,
        "infrared": False,
        "matrix": False,
        "max_kelvin": 9000,
        "min_ext_mz_firmware": 1532997580,
        "min_ext_mz_firmware_components": [2, 77],
        "min_kelvin": 1500,
        "multizone": True,
        "relays": False,
    },
    39: {
        "buttons": False,
        "chain": False,
        "color": False,
        "extended_multizone": False,
        "hev": False,
        "infrared": False,
        "matrix": False,
        "max_kelvin": 9000,
        "min_kelvin": 1500,
        "multizone": False,
        "relays": False,
    },
    40: {
        "buttons": False,
        "chain": False,
        "color": True,
        "extended_multizone": False,
        "hev": False,
        "infrared": False,
        "matrix": False,
        "max_kelvin": 9000,
        "min_kelvin": 1500,
        "multizone": False,
        "relays": False,
    },
    43: {
        "buttons": False,
        "chain": False,
        "color": True,
        "extended_multizone": False,
        "hev": False,
        "infrared": False,
        "matrix": False,
        "max_kelvin": 9000,
        "min_kelvin": 1500,
        "multizone": False,
        "relays": False,
    },
    44: {
        "buttons": False,
        "chain": False,
        "color": True,
        "extended_multizone": False,
        "hev": False,
        "infrared": False,
        "matrix": False,
        "max_kelvin": 9000,
        "min_kelvin": 1500,
        "multizone": False,
        "relays": False,
    },
    45: {
        "buttons": False,
        "chain": False,
        "color": True,
        "extended_multizone": False,
        "hev": False,
        "infrared": True,
        "matrix": False,
        "max_kelvin": 9000,
        "min_kelvin": 1500,
        "multizone": False,
        "relays": False,
    },
    46: {
        "buttons": False,
        "chain": False,
        "color": True,
        "extended_multizone": False,
        "hev": False,
        "infrared": True,
        "matrix": False,
        "max_kelvin": 9000,
        "min_kelvin": 1500,
        "multizone": False,
        "relays": False,
    },
    49: {
        "buttons": False,
        "chain": False,
        "color": True,
        "extended_multizone": False,
        "hev": False,
        "infrared": False,
        "matrix": False,
        "max_kelvin": 9000,
        "min_kelvin": 1500,
        "multizone": False,
        "relays": False,
    },
    50: {
        "buttons": False,
        "chain": False,
        "color": False,
        "extended_multizone": False,
        "hev": False,
        "infrared": False,
        "matrix": False,
        "max_kelvin": 9000,
        "min_kelvin": 1500,
        "multizone": False,
        "relays": False,
    },
    51: {
        "buttons": False,
        "chain": False,
        "color": False,
        "extended_multizone": False,
        "hev": False,
        "infrared": False,
        "matrix": False,
        "max_kelvin": 2700,
        "min_kelvin": 2700,
        "multizone": False,
        "relays": False,
    },
    52: {
        "buttons": False,
        "chain": False,
        "color": True,
        "extended_multizone": False,
        "hev": False,
        "infrared": False,
        "matrix": False,
        "max_kelvin": 9000,
        "min_kelvin": 1500,
        "multizone": False,
        "relays": False,
    },
    53: {
        "buttons": False,
        "chain": False,
        "color": True,
        "extended_multizone": False,
        "hev": False,
        "infrared": False,
        "matrix": False,
        "max_kelvin": 9000,
        "min_kelvin": 1500,
        "multizone": False,
        "relays": False,
    },
    55: {
        "buttons": False,
        "chain": True,
        "color": True,
        "extended_multizone": False,
        "hev": False,
        "infrared": False,
        "matrix": True,
        "max_kelvin": 9000,
        "min_kelvin": 2500,
        "multizone": False,
        "relays": False,
    },
    57: {
        "buttons": False,
        "chain": False,
        "color": True,
        "extended_multizone": False,
        "hev": False,
        "infrared": False,
        "matrix": True,
        "max_kelvin": 9000,
        "min_kelvin": 1500,
        "multizone": False,
        "relays": False,
    },
    59: {
        "buttons": False,
        "chain": False,
        "color": True,
        "extended_multizone": False,
        "hev": False,
        "infrared": False,
        "matrix": False,
        "max_kelvin": 9000,
        "min_kelvin": 1500,
        "multizone": False,
        "relays": False,
    },
    60: {
        "buttons": False,
        "chain": False,
        "color": False,
        "extended_multizone": False,
        "hev": False,
        "infrared": False,
        "matrix": False,
        "max_kelvin": 9000,
        "min_kelvin": 1500,
        "multizone": False,
        "relays": False,
    },
    61: {
        "buttons": False,
        "chain": False,
        "color": False,
        "extended_multizone": False,
        "hev": False,
        "infrared": False,
        "matrix": False,
        "max_kelvin": 2700,
        "min_kelvin": 2700,
        "multizone": False,
        "relays": False,
    },
    62: {
        "buttons": False,
        "chain": False,
        "color": True,
        "extended_multizone": False,
        "hev": False,
        "infrared": False,
        "matrix": False,
        "max_kelvin": 9000,
        "min_kelvin": 1500,
        "multizone": False,
        "relays": False,
    },
    63: {
        "buttons": False,
        "chain": False,
        "color": True,
        "extended_multizone": False,
        "hev": False,
        "infrared": False,
        "matrix": False,
        "max_kelvin": 9000,
        "min_kelvin": 1500,
        "multizone": False,
        "relays": False,
    },
    64: {
        "buttons": False,
        "chain": False,
        "color": True,
        "extended_multizone": False,
        "hev": False,
        "infrared": True,
        "matrix": False,
        "max_kelvin": 9000,
        "min_kelvin": 1500,
        "multizone": False,
        "relays": False,
    },
    65: {
        "buttons": False,
        "chain": False,
        "color": True,
        "extended_multizone": False,
        "hev": False,
        "infrared": True,
        "matrix": False,
        "max_kelvin": 9000,
        "min_kelvin": 1500,
        "multizone": False,
        "relays": False,
    },
    66: {
        "buttons": False,
        "chain": False,
        "color": False,
        "extended_multizone": False,
        "hev": False,
        "infrared": False,
        "matrix": False,
        "max_kelvin": 2700,
        "min_kelvin": 2700,
        "multizone": False,
        "relays": False,
    },
    68: {
        "buttons": False,
        "chain": False,
        "color": True,
        "extended_multizone": False,
        "hev": False,
        "infrared": False,
        "matrix": True,
        "max_kelvin": 9000,
        "min_kelvin": 1500,
        "multizone": False,
        "relays": False,
    },
    70: {
        "buttons": True,
        "chain": False,
        "color": False,
        "extended_multizone": False,
        "hev": False,
        "infrared": False,
        "matrix": False,
        "multizone": False,
        "relays": True,
        "temperature_range": None,
    },
    71: {
        "buttons": True,
        "chain": False,
        "color": False,
        "extended_multizone": False,
        "hev": False,
        "infrared": False,
        "matrix": False,
        "multizone": False,
        "relays": True,
        "temperature_range": None,
    },
    81: {
        "buttons": False,
        "chain": False,
        "color": False,
        "extended_multizone": False,
        "hev": False,
        "infrared": False,
        "matrix": False,
        "max_kelvin": 6500,
        "min_kelvin": 2200,
        "multizone": False,
        "relays": False,
    },
    82: {
        "buttons": False,
        "chain": False,
        "color": False,
        "extended_multizone": False,
        "hev": False,
        "infrared": False,
        "matrix": False,
        "max_kelvin": 2100,
        "min_kelvin": 2100,
        "multizone": False,
        "relays": False,
    },
    85: {
        "buttons": False,
        "chain": False,
        "color": False,
        "extended_multizone": False,
        "hev": False,
        "infrared": False,
        "matrix": False,
        "max_kelvin": 2000,
        "min_kelvin": 2000,
        "multizone": False,
        "relays": False,
    },
    87: {
        "buttons": False,
        "chain": False,
        "color": False,
        "extended_multizone": False,
        "hev": False,
        "infrared": False,
        "matrix": False,
        "max_kelvin": 2700,
        "min_kelvin": 2700,
        "multizone": False,
        "relays": False,
    },
    88: {
        "buttons": False,
        "chain": False,
        "color": False,
        "extended_multizone": False,
        "hev": False,
        "infrared": False,
        "matrix": False,
        "max_kelvin": 2700,
        "min_kelvin": 2700,
        "multizone": False,
        "relays": False,
    },
    89: {
        "buttons": True,
        "chain": False,
        "color": False,
        "extended_multizone": False,
        "hev": False,
        "infrared": False,
        "matrix": False,
        "multizone": False,
        "relays": True,
        "temperature_range": None,
    },
    90: {
        "buttons": False,
        "chain": False,
        "color": True,
        "extended_multizone": False,
        "hev": True,
        "infrared": False,
        "matrix": False,
        "max_kelvin": 9000,
        "min_kelvin": 1500,
        "multizone": False,
        "relays": False,
    },
    91: {
        "buttons": False,
        "chain": False,
        "color": True,
        "extended_multizone": False,
        "hev": False,
        "infrared": False,
        "matrix": False,
        "max_kelvin": 9000,
        "min_kelvin": 1500,
        "multizone": False,
        "relays": False,
    },
    92: {
        "buttons": False,
        "chain": False,
        "color": True,
        "extended_multizone": False,
        "hev": False,
        "infrared": False,
        "matrix": False,
        "max_kelvin": 9000,
        "min_kelvin": 1500,
        "multizone": False,
        "relays": False,
    },
    93: {
        "buttons": False,
        "chain": False,
        "color": True,
        "extended_multizone": False,
        "hev": False,
        "infrared": False,
        "matrix": False,
        "max_kelvin": 9000,
        "min_kelvin": 1500,
        "multizone": False,
        "relays": False,
    },
    94: {
        "buttons": False,
        "chain": False,
        "color": True,
        "extended_multizone": False,
        "hev": False,
        "infrared": False,
        "matrix": False,
        "max_kelvin": 9000,
        "min_kelvin": 1500,
        "multizone": False,
        "relays": False,
    },
    96: {
        "buttons": False,
        "chain": False,
        "color": False,
        "extended_multizone": False,
        "hev": False,
        "infrared": False,
        "matrix": False,
        "max_kelvin": 6500,
        "min_kelvin": 2200,
        "multizone": False,
        "relays": False,
    },
    97: {
        "buttons": False,
        "chain": False,
        "color": True,
        "extended_multizone": False,
        "hev": False,
        "infrared": False,
        "matrix": False,
        "max_kelvin": 9000,
        "min_kelvin": 1500,
        "multizone": False,
        "relays": False,
    },
    98: {
        "buttons": False,
        "chain": False,
        "color": True,
        "extended_multizone": False,
        "hev": False,
        "infrared": False,
        "matrix": False,
        "max_kelvin": 9000,
        "min_kelvin": 1500,
        "multizone": False,
        "relays": False,
    },
    99: {
        "buttons": False,
        "chain": False,
        "color": True,
        "extended_multizone": False,
        "hev": True,
        "infrared": False,
        "matrix": False,
        "max_kelvin": 9000,
        "min_kelvin": 1500,
        "multizone": False,
        "relays": False,
    },
    100: {
        "buttons": False,
        "chain": False,
        "color": False,
        "extended_multizone": False,
        "hev": False,
        "infrared": False,
        "matrix": False,
        "max_kelvin": 2100,
        "min_kelvin": 2100,
        "multizone": False,
        "relays": False,
    },
    101: {
        "buttons": False,
        "chain": False,
        "color": False,
        "extended_multizone": False,
        "hev": False,
        "infrared": False,
        "matrix": False,
        "max_kelvin": 2000,
        "min_kelvin": 2000,
        "multizone": False,
        "relays": False,
    },
    109: {
        "buttons": False,
        "chain": False,
        "color": True,
        "extended_multizone": False,
        "hev": False,
        "infrared": True,
        "matrix": False,
        "max_kelvin": 9000,
        "min_kelvin": 1500,
        "multizone": False,
        "relays": False,
    },
    110: {
        "buttons": False,
        "chain": False,
        "color": True,
        "extended_multizone": False,
        "hev": False,
        "infrared": True,
        "matrix": False,
        "max_kelvin": 9000,
        "min_kelvin": 1500,
        "multizone": False,
        "relays": False,
    },
    111: {
        "buttons": False,
        "chain": False,
        "color": True,
        "extended_multizone": False,
        "hev": False,
        "infrared": True,
        "matrix": False,
        "max_kelvin": 9000,
        "min_kelvin": 1500,
        "multizone": False,
        "relays": False,
    },
    112: {
        "buttons": False,
        "chain": False,
        "color": True,
        "extended_multizone": False,
        "hev": False,
        "infrared": True,
        "matrix": False,
        "max_kelvin": 9000,
        "min_kelvin": 1500,
        "multizone": False,
        "relays": False,
    },
    113: {
        "buttons": False,
        "chain": False,
        "color": False,
        "extended_multizone": False,
        "hev": False,
        "infrared": False,
        "matrix": False,
        "max_kelvin": 9000,
        "min_kelvin": 1500,
        "multizone": False,
        "relays": False,
    },
    114: {
        "buttons": False,
        "chain": False,
        "color": False,
        "extended_multizone": False,
        "hev": False,
        "infrared": False,
        "matrix": False,
        "max_kelvin": 9000,
        "min_kelvin": 1500,
        "multizone": False,
        "relays": False,
    },
    115: {
        "buttons": True,
        "chain": False,
        "color": False,
        "extended_multizone": False,
        "hev": False,
        "infrared": False,
        "matrix": False,
        "multizone": False,
        "relays": True,
        "temperature_range": None,
    },
    116: {
        "buttons": True,
        "chain": False,
        "color": False,
        "extended_multizone": False,
        "hev": False,
        "infrared": False,
        "matrix": False,
        "multizone": False,
        "relays": True,
        "temperature_range": None,
    },
    117: {
        "buttons": False,
        "chain": False,
        "color": True,
        "extended_multizone": True,
        "hev": False,
        "infrared": False,
        "matrix": False,
        "max_kelvin": 9000,
        "min_kelvin": 1500,
        "multizone": True,
        "relays": False,
    },
    118: {
        "buttons": False,
        "chain": False,
        "color": True,
        "extended_multizone": True,
        "hev": False,
        "infrared": False,
        "matrix": False,
        "max_kelvin": 9000,
        "min_kelvin": 1500,
        "multizone": True,
        "relays": False,
    },
    119: {
        "buttons": False,
        "chain": False,
        "color": True,
        "extended_multizone": True,
        "hev": False,
        "infrared": False,
        "matrix": False,
        "max_kelvin": 9000,
        "min_kelvin": 1500,
        "multizone": True,
        "relays": False,
    },
    120: {
        "buttons": False,
        "chain": False,
        "color": True,
        "extended_multizone": True,
        "hev": False,
        "infrared": False,
        "matrix": False,
        "max_kelvin": 9000,
        "min_kelvin": 1500,
        "multizone": True,
        "relays": False,
    },
    121: {
        "buttons": False,
        "chain": False,
        "color": True,
        "extended_multizone": False,
        "hev": False,
        "infrared": False,
        "matrix": False,
        "max_kelvin": 9000,
        "min_kelvin": 1500,
        "multizone": False,
        "relays": False,
    },
    122: {
        "buttons": False,
        "chain": False,
        "color": True,
        "extended_multizone": False,
        "hev": False,
        "infrared": False,
        "matrix": False,
        "max_kelvin": 9000,
        "min_kelvin": 1500,
        "multizone": False,
        "relays": False,
    },
    123: {
        "buttons": False,
        "chain": False,
        "color": True,
        "extended_multizone": False,
        "hev": False,
        "infrared": False,
        "matrix": False,
        "max_kelvin": 9000,
        "min_kelvin": 1500,
        "multizone": False,
        "relays": False,
    },
    124: {
        "buttons": False,
        "chain": False,
        "color": True,
        "extended_multizone": False,
        "hev": False,
        "infrared": False,
        "matrix": False,
        "max_kelvin": 9000,
        "min_kelvin": 1500,
        "multizone": False,
        "relays": False,
    },
    125: {
        "buttons": False,
        "chain": False,
        "color": False,
        "extended_multizone": False,
        "hev": False,
        "infrared": False,
        "matrix": False,
        "max_kelvin": 9000,
        "min_kelvin": 1500,
        "multizone": False,
        "relays": False,
    },
    126: {
        "buttons": False,
        "chain": False,
        "color": False,
        "extended_multizone": False,
        "hev": False,
        "infrared": False,
        "matrix": False,
        "max_kelvin": 9000,
        "min_kelvin": 1500,
        "multizone": False,
        "relays": False,
    },
    127: {
        "buttons": False,
        "chain": False,
        "color": False,
        "extended_multizone": False,
        "hev": False,
        "infrared": False,
        "matrix": False,
        "max_kelvin": 2700,
        "min_kelvin": 2700,
        "multizone": False,
        "relays": False,
    },
    128: {
        "buttons": False,
        "chain": False,
        "color": False,
        "extended_multizone": False,
        "hev": False,
        "infrared": False,
        "matrix": False,
        "max_kelvin": 2700,
        "min_kelvin": 2700,
        "multizone": False,
        "relays": False,
    },
    129: {
        "buttons": False,
        "chain": False,
        "color": True,
        "extended_multizone": False,
        "hev": False,
        "infrared": False,
        "matrix": False,
        "max_kelvin": 9000,
        "min_kelvin": 1500,
        "multizone": False,
        "relays": False,
    },
    130: {
        "buttons": False,
        "chain": False,
        "color": True,
        "extended_multizone": False,
        "hev": False,
        "infrared": False,
        "matrix": False,
        "max_kelvin": 9000,
        "min_kelvin": 1500,
        "multizone": False,
        "relays": False,
    },
    131: {
        "buttons": False,
        "chain": False,
        "color": False,
        "extended_multizone": False,
        "hev": False,
        "infrared": False,
        "matrix": False,
        "max_kelvin": 9000,
        "min_kelvin": 1500,
        "multizone": False,
        "relays": False,
    },
    132: {
        "buttons": False,
        "chain": False,
        "color": False,
        "extended_multizone": False,
        "hev": False,
        "infrared": False,
        "matrix": False,
        "max_kelvin": 9000,
        "min_kelvin": 1500,
        "multizone": False,
        "relays": False,
    },
    133: {
        "buttons": False,
        "chain": False,
        "color": False,
        "extended_multizone": False,
        "hev": False,
        "infrared": False,
        "matrix": False,
        "max_kelvin": 2700,
        "min_kelvin": 2700,
        "multizone": False,
        "relays": False,
    },
    134: {
        "buttons": False,
        "chain": False,
        "color": False,
        "extended_multizone": False,
        "hev": False,
        "infrared": False,
        "matrix": False,
        "max_kelvin": 2700,
        "min_kelvin": 2700,
        "multizone": False,
        "relays": False,
    },
    135: {
        "buttons": False,
        "chain": False,
        "color": True,
        "extended_multizone": False,
        "hev": False,
        "infrared": False,
        "matrix": False,
        "max_kelvin": 9000,
        "min_kelvin": 1500,
        "multizone": False,
        "relays": False,
    },
    136: {
        "buttons": False,
        "chain": False,
        "color": True,
        "extended_multizone": False,
        "hev": False,
        "infrared": False,
        "matrix": False,
        "max_kelvin": 9000,
        "min_kelvin": 1500,
        "multizone": False,
        "relays": False,
    },
    137: {
        "buttons": False,
        "chain": False,
        "color": True,
        "extended_multizone": False,
        "hev": False,
        "infrared": False,
        "matrix": True,
        "max_kelvin": 9000,
        "min_kelvin": 1500,
        "multizone": False,
        "relays": False,
    },
    138: {
        "buttons": False,
        "chain": False,
        "color": True,
        "extended_multizone": False,
        "hev": False,
        "infrared": False,
        "matrix": True,
        "max_kelvin": 9000,
        "min_kelvin": 1500,
        "multizone": False,
        "relays": False,
    },
    141: {
        "buttons": False,
        "chain": False,
        "color": True,
        "extended_multizone": True,
        "hev": False,
        "infrared": False,
        "matrix": False,
        "max_kelvin": 9000,
        "min_kelvin": 1500,
        "multizone": True,
        "relays": False,
    },
    142: {
        "buttons": False,
        "chain": False,
        "color": True,
        "extended_multizone": True,
        "hev": False,
        "infrared": False,
        "matrix": False,
        "max_kelvin": 9000,
        "min_kelvin": 1500,
        "multizone": True,
        "relays": False,
    },
    143: {
        "buttons": False,
        "chain": False,
        "color": True,
        "extended_multizone": True,
        "hev": False,
        "infrared": False,
        "matrix": False,
        "max_kelvin": 9000,
        "min_kelvin": 1500,
        "multizone": True,
        "relays": False,
    },
    144: {
        "buttons": False,
        "chain": False,
        "color": True,
        "extended_multizone": True,
        "hev": False,
        "infrared": False,
        "matrix": False,
        "max_kelvin": 9000,
        "min_kelvin": 1500,
        "multizone": True,
        "relays": False,
    },
    161: {
        "buttons": False,
        "chain": False,
        "color": True,
        "extended_multizone": True,
        "hev": False,
        "infrared": False,
        "matrix": False,
        "max_kelvin": 9000,
        "min_kelvin": 1500,
        "multizone": True,
        "relays": False,
    },
    162: {
        "buttons": False,
        "chain": False,
        "color": True,
        "extended_multizone": True,
        "hev": False,
        "infrared": False,
        "matrix": False,
        "max_kelvin": 9000,
        "min_kelvin": 1500,
        "multizone": True,
        "relays": False,
    },
    163: {
        "buttons": False,
        "chain": False,
        "color": True,
        "extended_multizone": False,
        "hev": False,
        "infrared": False,
        "matrix": False,
        "max_kelvin": 9000,
        "min_kelvin": 1500,
        "multizone": False,
        "relays": False,
    },
    164: {
        "buttons": False,
        "chain": False,
        "color": True,
        "extended_multizone": False,
        "hev": False,
        "infrared": False,
        "matrix": False,
        "max_kelvin": 9000,
        "min_kelvin": 1500,
        "multizone": False,
        "relays": False,
    },
    165: {
        "buttons": False,
        "chain": False,
        "color": True,
        "extended_multizone": False,
        "hev": False,
        "infrared": False,
        "matrix": False,
        "max_kelvin": 9000,
        "min_kelvin": 1500,
        "multizone": False,
        "relays": False,
    },
    166: {
        "buttons": False,
        "chain": False,
        "color": True,
        "extended_multizone": False,
        "hev": False,
        "infrared": False,
        "matrix": False,
        "max_kelvin": 9000,
        "min_kelvin": 1500,
        "multizone": False,
        "relays": False,
    },
    167: {
        "buttons": False,
        "chain": False,
        "color": True,
        "extended_multizone": False,
        "hev": False,
        "infrared": False,
        "matrix": False,
        "max_kelvin": 9000,
        "min_kelvin": 1500,
        "multizone": False,
        "relays": False,
    },
    168: {
        "buttons": False,
        "chain": False,
        "color": True,
        "extended_multizone": False,
        "hev": False,
        "infrared": False,
        "matrix": False,
        "max_kelvin": 9000,
        "min_kelvin": 1500,
        "multizone": False,
        "relays": False,
    },
    169: {
        "buttons": False,
        "chain": False,
        "color": True,
        "extended_multizone": False,
        "hev": False,
        "infrared": False,
        "matrix": False,
        "max_kelvin": 9000,
        "min_kelvin": 1500,
        "multizone": False,
        "relays": False,
    },
    170: {
        "buttons": False,
        "chain": False,
        "color": True,
        "extended_multizone": False,
        "hev": False,
        "infrared": False,
        "matrix": False,
        "max_kelvin": 9000,
        "min_kelvin": 1500,
        "multizone": False,
        "relays": False,
    },
    171: {
        "buttons": False,
        "chain": False,
        "color": True,
        "extended_multizone": False,
        "hev": False,
        "infrared": False,
        "matrix": True,
        "max_kelvin": 9000,
        "min_kelvin": 1500,
        "multizone": False,
        "relays": False,
    },
    173: {
        "buttons": False,
        "chain": False,
        "color": True,
        "extended_multizone": False,
        "hev": False,
        "infrared": False,
        "matrix": True,
        "max_kelvin": 9000,
        "min_kelvin": 1500,
        "multizone": False,
        "relays": False,
    },
    174: {
        "buttons": False,
        "chain": False,
        "color": True,
        "extended_multizone": False,
        "hev": False,
        "infrared": False,
        "matrix": True,
        "max_kelvin": 9000,
        "min_kelvin": 1500,
        "multizone": False,
        "relays": False,
    },
    175: {
        "buttons": False,
        "chain": False,
        "color": True,
        "extended_multizone": False,
        "hev": False,
        "infrared": False,
        "matrix": False,
        "max_kelvin": 9000,
        "min_kelvin": 1500,
        "multizone": False,
        "relays": False,
    },
    176: {
        "buttons": False,
        "chain": False,
        "color": True,
        "extended_multizone": False,
        "hev": False,
        "infrared": False,
        "matrix": True,
        "max_kelvin": 9000,
        "min_kelvin": 1500,
        "multizone": False,
        "relays": False,
    },
    177: {
        "buttons": False,
        "chain": False,
        "color": True,
        "extended_multizone": False,
        "hev": False,
        "infrared": False,
        "matrix": True,
        "max_kelvin": 9000,
        "min_kelvin": 1500,
        "multizone": False,
        "relays": False,
    },
    178: {
        "buttons": False,
        "chain": False,
        "color": True,
        "extended_multizone": False,
        "hev": False,
        "infrared": False,
        "matrix": False,
        "max_kelvin": 9000,
        "min_kelvin": 1500,
        "multizone": False,
        "relays": False,
    },
    179: {
        "buttons": False,
        "chain": False,
        "color": True,
        "extended_multizone": False,
        "hev": False,
        "infrared": False,
        "matrix": False,
        "max_kelvin": 9000,
        "min_kelvin": 1500,
        "multizone": False,
        "relays": False,
    },
    180: {
        "buttons": False,
        "chain": False,
        "color": True,
        "extended_multizone": False,
        "hev": False,
        "infrared": False,
        "matrix": False,
        "max_kelvin": 9000,
        "min_kelvin": 1500,
        "multizone": False,
        "relays": False,
    },
    181: {
        "buttons": False,
        "chain": False,
        "color": True,
        "extended_multizone": False,
        "hev": False,
        "infrared": False,
        "matrix": False,
        "max_kelvin": 9000,
        "min_kelvin": 1500,
        "multizone": False,
        "relays": False,
    },
    182: {
        "buttons": False,
        "chain": False,
        "color": True,
        "extended_multizone": False,
        "hev": False,
        "infrared": False,
        "matrix": False,
        "max_kelvin": 9000,
        "min_kelvin": 1500,
        "multizone": False,
        "relays": False,
    },
    185: {
        "buttons": False,
        "chain": False,
        "color": True,
        "extended_multizone": False,
        "hev": False,
        "infrared": False,
        "matrix": True,
        "max_kelvin": 9000,
        "min_kelvin": 1500,
        "multizone": False,
        "relays": False,
    },
    186: {
        "buttons": False,
        "chain": False,
        "color": True,
        "extended_multizone": False,
        "hev": False,
        "infrared": False,
        "matrix": True,
        "max_kelvin": 9000,
        "min_kelvin": 1500,
        "multizone": False,
        "relays": False,
    },
    187: {
        "buttons": False,
        "chain": False,
        "color": True,
        "extended_multizone": False,
        "hev": False,
        "infrared": False,
        "matrix": False,
        "max_kelvin": 9000,
        "min_kelvin": 1500,
        "multizone": False,
        "relays": False,
    },
    188: {
        "buttons": False,
        "chain": False,
        "color": True,
        "extended_multizone": False,
        "hev": False,
        "infrared": False,
        "matrix": False,
        "max_kelvin": 9000,
        "min_kelvin": 1500,
        "multizone": False,
        "relays": False,
    },
    201: {
        "buttons": False,
        "chain": False,
        "color": True,
        "extended_multizone": False,
        "hev": False,
        "infrared": False,
        "matrix": True,
        "max_kelvin": 9000,
        "min_kelvin": 1500,
        "multizone": False,
        "relays": False,
    },
    202: {
        "buttons": False,
        "chain": False,
        "color": True,
        "extended_multizone": False,
        "hev": False,
        "infrared": False,
        "matrix": True,
        "max_kelvin": 9000,
        "min_kelvin": 1500,
        "multizone": False,
        "relays": False,
    },
    203: {
        "buttons": False,
        "chain": False,
        "color": True,
        "extended_multizone": True,
        "hev": False,
        "infrared": False,
        "matrix": False,
        "max_kelvin": 9000,
        "min_kelvin": 1500,
        "multizone": True,
        "relays": False,
    },
    204: {
        "buttons": False,
        "chain": False,
        "color": True,
        "extended_multizone": True,
        "hev": False,
        "infrared": False,
        "matrix": False,
        "max_kelvin": 9000,
        "min_kelvin": 1500,
        "multizone": True,
        "relays": False,
    },
    205: {
        "buttons": False,
        "chain": False,
        "color": True,
        "extended_multizone": True,
        "hev": False,
        "infrared": False,
        "matrix": False,
        "max_kelvin": 9000,
        "min_kelvin": 1500,
        "multizone": True,
        "relays": False,
    },
    206: {
        "buttons": False,
        "chain": False,
        "color": True,
        "extended_multizone": True,
        "hev": False,
        "infrared": False,
        "matrix": False,
        "max_kelvin": 9000,
        "min_kelvin": 1500,
        "multizone": True,
        "relays": False,
    },
    213: {
        "buttons": False,
        "chain": False,
        "color": True,
        "extended_multizone": True,
        "hev": False,
        "infrared": False,
        "matrix": False,
        "max_kelvin": 9000,
        "min_kelvin": 1500,
        "multizone": True,
        "relays": False,
    },
    214: {
        "buttons": False,
        "chain": False,
        "color": True,
        "extended_multizone": True,
        "hev": False,
        "infrared": False,
        "matrix": False,
        "max_kelvin": 9000,
        "min_kelvin": 1500,
        "multizone": True,
        "relays": False,
    },
    215: {
        "buttons": False,
        "chain": False,
        "color": True,
        "extended_multizone": False,
        "hev": False,
        "infrared": False,
        "matrix": True,
        "max_kelvin": 9000,
        "min_kelvin": 1500,
        "multizone": False,
        "relays": False,
    },
    216: {
        "buttons": False,
        "chain": False,
        "color": True,
        "extended_multizone": False,
        "hev": False,
        "infrared": False,
        "matrix": True,
        "max_kelvin": 9000,
        "min_kelvin": 1500,
        "multizone": False,
        "relays": False,
    },
    217: {
        "buttons": False,
        "chain": False,
        "color": True,
        "extended_multizone": False,
        "hev": False,
        "infrared": False,
        "matrix": True,
        "max_kelvin": 9000,
        "min_kelvin": 1500,
        "multizone": False,
        "relays": False,
    },
    218: {
        "buttons": False,
        "chain": False,
        "color": True,
        "extended_multizone": False,
        "hev": False,
        "infrared": False,
        "matrix": True,
        "max_kelvin": 9000,
        "min_kelvin": 1500,
        "multizone": False,
        "relays": False,
    },
    219: {
        "buttons": False,
        "chain": False,
        "color": True,
        "extended_multizone": False,
        "hev": False,
        "infrared": False,
        "matrix": True,
        "max_kelvin": 9000,
        "min_kelvin": 1500,
        "multizone": False,
        "relays": False,
    },
    220: {
        "buttons": False,
        "chain": False,
        "color": True,
        "extended_multizone": False,
        "hev": False,
        "infrared": False,
        "matrix": True,
        "max_kelvin": 9000,
        "min_kelvin": 1500,
        "multizone": False,
        "relays": False,
    },
    221: {
        "buttons": False,
        "chain": False,
        "color": True,
        "extended_multizone": False,
        "hev": False,
        "infrared": False,
        "matrix": True,
        "max_kelvin": 9000,
        "min_kelvin": 1500,
        "multizone": False,
        "relays": False,
    },
    222: {
        "buttons": False,
        "chain": False,
        "color": True,
        "extended_multizone": False,
        "hev": False,
        "infrared": False,
        "matrix": True,
        "max_kelvin": 9000,
        "min_kelvin": 1500,
        "multizone": False,
        "relays": False,
    },
    223: {
        "buttons": False,
        "chain": False,
        "color": True,
        "extended_multizone": False,
        "hev": False,
        "infrared": False,
        "matrix": False,
        "max_kelvin": 9000,
        "min_kelvin": 1500,
        "multizone": False,
        "relays": False,
    },
    224: {
        "buttons": False,
        "chain": False,
        "color": True,
        "extended_multizone": False,
        "hev": False,
        "infrared": False,
        "matrix": False,
        "max_kelvin": 9000,
        "min_kelvin": 1500,
        "multizone": False,
        "relays": False,
    },
    225: {
        "buttons": False,
        "chain": False,
        "color": True,
        "extended_multizone": False,
        "hev": False,
        "infrared": False,
        "matrix": False,
        "max_kelvin": 9000,
        "min_kelvin": 1500,
        "multizone": False,
        "relays": False,
    },
}



# message.py
# Author: Meghan Clark

import struct
import binascii
import sys

BROADCAST_MAC = "00:00:00:00:00:00"
BROADCAST_SOURCE_ID = 0

HEADER_SIZE_BYTES = 36


class Message(object):
    def __init__(
        self,
        msg_type,
        target_addr,
        source_id,
        seq_num,
        ack_requested=False,
        response_requested=False,
    ):
        # Frame
        self.frame_format = ["<H", "<H", "<L"]
        self.size = None  # 16 bits/uint16
        self.origin = 0  # 2 bits/uint8, must be zero
        self.tagged = (
            1 if target_addr == BROADCAST_MAC else 0
        )  # 1 bit/bool, also must be one if getservice
        self.addressable = 1  # 1 bit/bool, must be one
        self.protocol = 1024  # 12 bits/uint16
        self.source_id = source_id  # 32 bits/uint32, unique ID set by client. If zero, broadcast reply requested. If non-zero, unicast reply requested.

        # Frame Address
        self.frame_addr_format = ["<Q", "<BBBBBB", "<B", "<B"]
        self.target_addr = target_addr  # 64 bits/uint64, either single MAC address or all zeroes for broadcast.
        self.reserved = 0  # 48 bits/uint8 x 6, all zero
        self.reserved = 0  # 6 bits, all zero
        self.ack_requested = 1 if ack_requested else 0  # 1 bit/bool, 1 = yes
        self.response_requested = 1 if response_requested else 0  # 1 bit/bool, 1 = yes
        self.seq_num = seq_num  # 8 bits/uint8, wraparound

        # Protocol Header
        self.protocol_header_format = ["<Q", "<H", "<H"]
        self.reserved = 0  # 64 bits/uint64, all zero
        self.message_type = msg_type  # 16 bits/uint16
        self.reserved = 0  # 16 bits/uint16, all zero

        self.payload_fields = []  # tuples of ("label", value)

        self._packed_message = None

    @property
    def packed_message(self):
        if self._packed_message is None:
            self._packed_message = self.generate_packed_message()
        return self._packed_message

    @packed_message.setter
    def packed_message(self, value):
        self._packed_message = value

    def generate_packed_message(self):
        self.payload = self.get_payload()
        self.header = self.get_header()
        packed_message = self.header + self.payload
        return packed_message

    # frame (and thus header) needs to be generated after payload (for size field)
    def get_header(self):
        if self.size == None:
            self.size = self.get_msg_size()
        frame_addr = self.get_frame_addr()
        frame = self.get_frame()
        protocol_header = self.get_protocol_header()
        header = frame + frame_addr + protocol_header
        return header

    # Default: No payload unless method overridden
    def get_payload(self):
        return struct.pack("")

    def get_frame(self):
        size_format = self.frame_format[0]
        flags_format = self.frame_format[1]
        source_id_format = self.frame_format[2]
        size = struct.pack(size_format, self.size)
        flags = struct.pack(
            flags_format,
            ((self.origin & 0b11) << 14)
            | ((self.tagged & 0b1) << 13)
            | ((self.addressable & 0b1) << 12)
            | (self.protocol & 0b111111111111),
        )
        source_id = struct.pack(source_id_format, self.source_id)
        frame = size + flags + source_id
        return frame

    def get_frame_addr(self):
        mac_addr_format = self.frame_addr_format[0]
        reserved_48_format = self.frame_addr_format[1]
        response_flags_format = self.frame_addr_format[2]
        seq_num_format = self.frame_addr_format[3]
        mac_addr = struct.pack(mac_addr_format, convert_MAC_to_int(self.target_addr))
        reserved_48 = struct.pack(reserved_48_format, *([self.reserved] * 6))
        response_flags = struct.pack(
            response_flags_format,
            ((self.reserved & 0b111111) << 2)
            | ((self.ack_requested & 0b1) << 1)
            | (self.response_requested & 0b1),
        )
        seq_num = struct.pack(seq_num_format, self.seq_num)
        frame_addr = mac_addr + reserved_48 + response_flags + seq_num
        return frame_addr

    def get_protocol_header(self):
        reserved_64_format = self.protocol_header_format[0]
        message_type_format = self.protocol_header_format[1]
        reserved_16_format = self.protocol_header_format[2]
        reserved_64 = struct.pack(reserved_64_format, self.reserved)
        message_type = struct.pack(message_type_format, self.message_type)
        reserved_16 = struct.pack(reserved_16_format, self.reserved)
        protocol_header = reserved_64 + message_type + reserved_16
        return protocol_header

    def get_msg_size(self):
        payload_size_bytes = len(self.payload)
        return HEADER_SIZE_BYTES + payload_size_bytes

    def __str__(self):
        indent = "  "
        s = self.__class__.__name__ + "\n"
        s += indent + "Size: {}\n".format(self.size)
        s += indent + "Origin: {}\n".format(self.origin)
        s += indent + "Tagged: {}\n".format(self.tagged)
        s += indent + "Protocol: {}\n".format(self.protocol)
        s += indent + "Source ID: {}\n".format(self.source_id)
        s += indent + "Target MAC Address: {}\n".format(self.target_addr)
        s += indent + "Ack Requested: {}\n".format(self.ack_requested)
        s += indent + "Response Requested: {}\n".format(self.response_requested)
        s += indent + "Seq Num: {}\n".format(self.seq_num)
        s += indent + "Message Type: {}\n".format(self.message_type)
        s += indent + "Payload:"
        for field in self.payload_fields:
            s += "\n" + indent * 2 + "{}: {}".format(field[0], field[1])
        if len(self.payload_fields) == 0:
            s += "\n" + indent * 2 + "<empty>"
        s += "\n"
        s += indent + "Bytes:\n"
        s += indent * 2 + str(
            [
                hex(b)
                for b in struct.unpack(
                    "B" * (len(self.packed_message)), self.packed_message
                )
            ]
        )
        s += "\n"
        return s


# reverses bytes for little endian, then converts to int
def convert_MAC_to_int(addr):
    reverse_bytes_str = addr.split(":")
    reverse_bytes_str.reverse()
    addr_str = "".join(reverse_bytes_str)
    return int(addr_str, 16)


def little_endian(bs):
    return bytes(reversed(bs.bytes))
